SystemOrganization addCategory: #'Seaside-Gettext-Core'!

WAObject subclass: #WALazyTransaltedString
	instanceVariableNames: 'key'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Gettext-Core'!

!WALazyTransaltedString class methodsFor: 'instance creation' stamp: 'pmm 5/21/2011 16:43'!
key: aString
	^ self basicNew initializeWithKey: aString! !

!WALazyTransaltedString methodsFor: 'initialization' stamp: 'pmm 5/21/2011 16:43'!
initializeWithKey: aString
	self initialize.
	key := aString! !

!WALazyTransaltedString methodsFor: 'printing' stamp: 'pmm 5/21/2011 16:47'!
printOn: aStream
	key seasideTranslated printOn: aStream! !

!WALazyTransaltedString methodsFor: 'rendering' stamp: 'pmm 5/21/2011 16:45'!
renderOn: html
	html translate: key! !

WAObject subclass: #WALocalizationContext
	instanceVariableNames: 'domain translator localeId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Gettext-Core'!

!WALocalizationContext methodsFor: 'accessing' stamp: 'pmm 5/8/2010 21:41'!
defaultApplicationName
	^ 'defaultApplication'! !

!WALocalizationContext methodsFor: 'accessing' stamp: 'pmm 5/8/2010 22:13'!
domain
	domain isNil ifTrue: [
		"determine the domain name
		1. try if it is configured on the application
		2. use the application name
		3. send #defaultApplicationName"
		| configuredDomain |
		configuredDomain := [ self application gettextDomain ]
			on: WAAttributeNotFound
			do: [ :error | nil ].
		domain := configuredDomain isEmptyOrNil
			ifFalse: [ configuredDomain ]
			ifTrue: [
				| applicationName |
				applicationName := self application name.
				applicationName isEmptyOrNil
					ifFalse: [ applicationName ]
					ifTrue: [ self defaultApplicationName ] ] ].
	^ domain! !

!WALocalizationContext methodsFor: 'accessing' stamp: 'pmm 5/10/2010 22:37'!
fistSupportedLocaleWithFallback: aNiladicBlock
	| availableLanguageLocaleIDs |
	availableLanguageLocaleIDs := GetTextTranslator availableLanguageLocaleIDs.
	self requestContext request acceptLanguage do: [ :each |
		| locale |
		locale := each value.
		(availableLanguageLocaleIDs includes: locale asLocaleID)
			ifTrue: [ ^ locale ] ].
	^ aNiladicBlock value! !

!WALocalizationContext methodsFor: 'accessing' stamp: 'pmm 5/10/2010 22:38'!
localeId
	localeId isNil ifTrue: [
		| session |
		session := self session.
		self session notNil
			ifTrue: [
				"if there's a session take it from the session"
				localeId := session locale asLocaleID ]
			ifFalse: [
				"do language negotiation and take the first supported one"
				self requestContext acceptLanguage do: [ :each |
					| candidate |
					candidate := each value asLocaleID.
					GetTextTranslator translators
						at: candidate
						ifPresent: [ :t |
							translator := t.
							^ localeId := candidate ] ].
				"fall back to default"
				translator := GetTextTranslator default.
				localeId := translator localeID ] ].
	^ localeId! !

!WALocalizationContext methodsFor: 'translating' stamp: 'pmm 5/8/2010 21:50'!
translate: aString
	^ self translator
		translate: aString
		inDomain: self domain! !

!WALocalizationContext methodsFor: 'translating' stamp: 'pmm 5/8/2010 21:47'!
translate: aString to: aLocale
	^ (GetTextTranslator availableForLocaleID: aLocale asLocaleID)
		translate: aString
		inDomain: self domain! !

!WALocalizationContext methodsFor: 'accessing' stamp: 'pmm 5/8/2010 22:05'!
translator
	translator isNil ifTrue: [
		translator := GetTextTranslator availableForLocaleID: self localeId ].
	^ translator! !

WAObject subclass: #WATranslatedArgumentsFinder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Gettext-Core'!

!WATranslatedArgumentsFinder methodsFor: 'accessing' stamp: 'pmm 5/21/2011 17:10'!
translatedArguments
	"self new translatedArguments"
	| searcher currentClass result innerMatchHandler extractFirstArgument extractReceiver |
	currentClass := WAValueHolder with: nil.
	result := GROrderedMultiMap new.
	innerMatchHandler := [ :node :literal |
		| key |
		key := MethodReference
			class: currentClass contents
			selector: node methodNode selector.
			result at: key add: literal value ].
	extractFirstArgument := [ :node :answer |
		| firstArgument |
		firstArgument := node arguments first.
		firstArgument isLiteral ifTrue: [
			innerMatchHandler value: node value: firstArgument ] ].
	extractReceiver := [ :node :answer |
		node receiver isLiteral ifTrue: [
			innerMatchHandler value: node value: node receiver ] ].
	searcher := RBParseTreeSearcher new
		matches: '`@object translate: `#string' do: extractFirstArgument;
		matches: '`@object translate: `#string to: `@arg' do: extractFirstArgument;
		matches: '`#string seasideTranslated' do: extractReceiver;
		matches: '`#string seasideLazyTranslated' do: extractReceiver;
		matches: '`#string seasideTranslatedTo: `@arg' do: extractReceiver;
		yourself.
	BrowserEnvironment new classesAndSelectorsDo: [ :class :selector |
		(searcher canMatchMethod: class >> selector) ifTrue: [
		| tree |
		(tree := class parseTreeFor: selector) notNil ifTrue: [
			currentClass contents: class.
			searcher executeTree: tree ] ] ].
	^ Array streamContents: [ :stream |
		result keysDo: [ :key |
			stream nextPut: key -> (result allAt: key) ] ]! !

!String methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/21/2011 17:09'!
seasideLazyTranslated
	^ WALazyTransaltedString key: self! !

!String methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/15/2010 21:46'!
seasideTranslated
	^ WACurrentLocalizationContext value translate: self! !

!String methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/15/2010 21:47'!
seasideTranslatedTo: aLocale
	^ WACurrentLocalizationContext value translate: self to: aLocale! !

GetTextExporter subclass: #WAGetTextExporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Gettext-Core'!

!WAGetTextExporter methodsFor: 'exporting' stamp: 'pmm 5/8/2010 13:52'!
appendStringArtumentsInto: domains
	| found |
	
	found := WATranslatedArgumentsFinder new translatedArguments.
	found do: [ :assoc |
		| literals domainName methodReference keywords |
		methodReference := assoc key.
		keywords := assoc value.
		domainName := self getTextDomainForClassCategory:
			(Smalltalk at: methodReference classSymbol) category.
		literals := domains at: domainName ifAbsentPut: [ Dictionary new ].
		keywords do: [ :literal |
			| references |
			references := literals at: literal ifAbsentPut: [ OrderedCollection new ].
			references add: methodReference ] ]. 
! !

!WAGetTextExporter methodsFor: 'exporting' stamp: 'pmm 5/8/2010 13:52'!
appendTranslations: domains
	self appendStringArtumentsInto: domains! !

!WALocale methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/7/2010 17:46'!
asLocaleID
	^ LocaleID
		isoLanguage: self language
		isoCountry: self country! !

!WARenderer methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/8/2010 16:09'!
translate: aString
	self text: (WACurrentLocalizationContext value translate: aString)! !

!WARenderer methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/8/2010 16:09'!
translate: aString to: aLocale
	self text: (WACurrentLocalizationContext value translate: aString to: aLocale)! !

!WASession methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/10/2010 22:36'!
defaultLocale
	^ WACurrentLocalizationContext value
		fistSupportedLocaleWithFallback:  [ self fallbackLocale ]! !

!WASession methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/10/2010 22:25'!
fallbackLocale
	^ WALocale
		language: 'en'! !

!WASession methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/7/2010 17:37'!
locale
	^ self properties
		at: #gettextLocale
		ifAbsentPut: [ self defaultLocale ]! !

!WASession methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/8/2010 20:57'!
locale: aLocale
	self properties
		at: #gettextLocale
		put: aLocale! !

WARequestFilter subclass: #WALocalizationContextFilter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Gettext-Core'!

!WALocalizationContextFilter methodsFor: 'processing' stamp: 'pmm 5/7/2010 17:33'!
handleFiltered: aRequestContext 
	WACurrentLocalizationContext
		use: WALocalizationContext new
		during: [ super handleFiltered: aRequestContext ]! !

!GRPackage class methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/8/2010 15:42'!
seasideGettextCore
	^ self new
		name: 'Seaside-Gettext-Core';
		description: 'i18n and l10n using gettext';
		url: #seasideAddonsUrl;
		yourself! !

!WAApplication methodsFor: '*seaside-gettext-core' stamp: 'pmm 5/8/2010 22:07'!
gettextDomain
	^ self preferenceAt: #gettextDomain! !

WADynamicVariable subclass: #WACurrentLocalizationContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Gettext-Core'!

WASystemConfiguration subclass: #WAGetTextConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Gettext-Core'!

!WAGetTextConfiguration methodsFor: 'description' stamp: 'pmm 5/8/2010 21:59'!
describeOn: config
	(config string: #gettextDomain)
		label: 'Gettext Domain';
		comment: 'The name of the domain where all lozalized strings should come from.'! !
